﻿using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;



namespace Ato.CD.Inbound.GetPaymentPlan202003Validator
{

    public class ControlMetaData 
    {
        public string RequestedService { get; set; }
        public string MessageSourceID { get; set; }
        public string UserLoginText { get; set; }
        public string QualityOfServiceIndicator { get; set; }
        public bool SetDefaultUserSecurityAttributeFlags { get; set; }
        public bool SetDefaultDataSecurityAttributeFlags { get; set; }
        public bool SetNonInteractiveFlag { get; set; }
        public bool SetMessageDatetime { get; set; }
        public string RelatedProcID { get; set; }
        public string RelatedProcRunID { get; set; }
        public bool SetFormValidateFlag { get; set; }


    }
    /// comment test CI build 
    /// <summary>
    /// Common utility methods
    /// </summary>
    public static class Common
    {
        internal const string EAINamespacePrefix = "default";
        internal const string EAINamespace = "http://www.ato.gov.au/Subjects/EAI/2005/09/ATOCanonical";

        

        public static void AddControlHeaders(XmlDocument transformedRequestXmlDocument, XmlNamespaceManager namespaceManager,
            ControlMetaData controlMetaData, string receiptNo)
        {
            const string ControlPath = "default:Control";
            const string RequestedServiceElementName = "RequestedService";
            const string MessageSourceIDElementName = "MessageSourceID";
            const string QualityOfServiceIndicatorElementName = "QualityOfServiceIndicator";
            const string UserLoginTextElementName = "UserLoginText";
            const string MessageDateTimeElementName = "MessageDatetime";
            const string TransactionId = "TransactionId";

            XmlNode controlElement;
            XmlNode newNode;
            XmlNode lastCreatedNode;

            if (transformedRequestXmlDocument != null)
            {
                controlElement = transformedRequestXmlDocument.DocumentElement.SelectSingleNode(ControlPath, namespaceManager);
                if (controlElement != null)
                {
                    newNode = transformedRequestXmlDocument.CreateElement(RequestedServiceElementName, EAINamespace);
                    newNode.InnerText = controlMetaData.RequestedService;
                    lastCreatedNode = controlElement.InsertBefore(newNode, controlElement.FirstChild);

                    newNode = transformedRequestXmlDocument.CreateElement(MessageSourceIDElementName, EAINamespace);
                    newNode.InnerText = controlMetaData.MessageSourceID;

                    lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);


                    if (controlMetaData.QualityOfServiceIndicator != null)
                    {
                        newNode = transformedRequestXmlDocument.CreateElement(QualityOfServiceIndicatorElementName, EAINamespace);
                        newNode.InnerText = controlMetaData.QualityOfServiceIndicator;
                        lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);
                    }

                    newNode = transformedRequestXmlDocument.CreateElement(UserLoginTextElementName, EAINamespace);
                    newNode.InnerText = controlMetaData.UserLoginText;

                    lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);


                    if (controlMetaData.SetDefaultUserSecurityAttributeFlags)
                    {
                        lastCreatedNode = CreateUserSecurityTags(transformedRequestXmlDocument, controlElement, lastCreatedNode);
                    }
                    if (controlMetaData.SetNonInteractiveFlag)
                    {
                        lastCreatedNode = CreateTAFlags(transformedRequestXmlDocument, controlElement, lastCreatedNode, controlMetaData.SetFormValidateFlag);
                    }

                    if (controlMetaData.SetDefaultDataSecurityAttributeFlags)
                    {
                        lastCreatedNode = CreateDataSecurityTags(transformedRequestXmlDocument, controlElement, lastCreatedNode);
                    }


                    if (controlMetaData.SetMessageDatetime)
                    {
                        newNode = transformedRequestXmlDocument.CreateElement(MessageDateTimeElementName, EAINamespace);
                        newNode.InnerText = DateTime.Now.ToString("yyyy-MM-ddTHH:mm:sszzz");
                        lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);
                    }
                    newNode = transformedRequestXmlDocument.CreateElement(TransactionId, EAINamespace);
                    newNode.InnerText = receiptNo;
                }

            }
        }

        private static XmlNode CreateUserSecurityTags(XmlDocument transformedRequestXmlDocument, XmlNode controlElement, XmlNode lastCreatedNode)
        {
            const string UserSecurityAttributeFlagsElementName = "UserSecurityAttributeFlags";

            XmlNode newNode;
            XmlNode childNode;

            newNode = transformedRequestXmlDocument.CreateElement(UserSecurityAttributeFlagsElementName, EAINamespace);
            lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);

            childNode = transformedRequestXmlDocument.CreateElement("UnClassifiedIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("TopSecretIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("SecretIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("ConfidentialIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("HighlyProtectedIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("ProtectedIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("InConfidenceIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("CharacterClearanceIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("StaffIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("SpecialInterestClientIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("InternalIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("TaxPractitionerIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            childNode = transformedRequestXmlDocument.CreateElement("BusinessIndicator", EAINamespace);
            childNode.InnerText = "1";
            newNode.AppendChild(childNode);

            for (int i = 1; i < 5; i++)
            {
                childNode = transformedRequestXmlDocument.CreateElement("EmployeeLevelIndicator" + i.ToString(), EAINamespace);
                childNode.InnerText = "1";
                newNode.AppendChild(childNode);
            }

            for (int i = 1; i < 16; i++)
            {
                childNode = transformedRequestXmlDocument.CreateElement("SecurityIndicator" + i.ToString(), EAINamespace);
                childNode.InnerText = "1";
                newNode.AppendChild(childNode);
            }

            return lastCreatedNode;

        }

        private static XmlNode CreateTAFlags(XmlDocument transformedRequestXmlDocument, XmlNode controlElement, XmlNode lastCreatedNode, bool setFormValidateFlag)
        {
            const string TAFlagsElementName = "TAFlags";

            XmlNode newNode;
            XmlNode childNode;

            newNode = transformedRequestXmlDocument.CreateElement(TAFlagsElementName, EAINamespace);
            lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);

            childNode = transformedRequestXmlDocument.CreateElement("NonInteractiveIndicator", EAINamespace);
            childNode.InnerText = "0";
            newNode.AppendChild(childNode);



            if (setFormValidateFlag)
            {
                childNode = transformedRequestXmlDocument.CreateElement("FormValidateIndicator", EAINamespace);
                childNode.InnerText = "0";
                newNode.AppendChild(childNode);
            }


            return lastCreatedNode;

        }

        private static XmlNode CreateDataSecurityTags(XmlDocument transformedRequestXmlDocument, XmlNode controlElement, XmlNode lastCreatedNode)
        {
            const string DataSecurityFlagsElementName = "DataSecurityFlags";

            XmlNode newNode;
            XmlNode childNode;

            newNode = transformedRequestXmlDocument.CreateElement(DataSecurityFlagsElementName, EAINamespace);
            lastCreatedNode = controlElement.InsertAfter(newNode, lastCreatedNode);

            for (int i = 1; i < 51; i++)
            {
                childNode = transformedRequestXmlDocument.CreateElement("DataSecurityIndicator" + i.ToString(), EAINamespace);
                childNode.InnerText = "1";
                newNode.AppendChild(childNode);
            }

            return lastCreatedNode;

        }

        public static T ToType<T>(string input, string defaultnamespace) where T : class
        {
            XmlSerializer ser = new XmlSerializer(typeof(T), defaultnamespace);

            using (StringReader sr = new StringReader(input))
                return (T)ser.Deserialize(sr);
        }




    }



}
